@@ -1,7 +1,7 @@
 //
 // USB device support code for the Printer Application Framework
 //
-// Copyright © 2019-2020 by Michael R Sweet.
+// Copyright © 2019-2021 by Michael R Sweet.
 // Copyright © 2007-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -12,6 +12,7 @@
 // Include necessary headers...
 //
 
+//#define DEBUG 1
 #include "device-private.h"
 #include "printer.h"
 #ifdef HAVE_LIBUSB
@@ -225,6 +226,8 @@ pappl_usb_find(
 	_PAPPL_DEBUG("pappl_usb_find:             num_altsetting=%d\n", ifaceptr->num_altsetting);
 	_PAPPL_DEBUG("pappl_usb_find:             altsetting=%p\n", ifaceptr->altsetting);
 
+        device->protocol = 0;
+
 	for (altset = 0, altptr = ifaceptr->altsetting; (int)altset < ifaceptr->num_altsetting; altset ++, altptr ++)
 	{
 	  _PAPPL_DEBUG("pappl_usb_find:             altset%d - bInterfaceClass=%d, bInterfaceSubClass=%d, bInterfaceProtocol=%d\n", altset, altptr->bInterfaceClass, altptr->bInterfaceSubClass, altptr->bInterfaceProtocol);
@@ -235,7 +238,7 @@ pappl_usb_find(
 	  if (altptr->bInterfaceProtocol != 1 && altptr->bInterfaceProtocol != 2)
 	    continue;
 
-	  if (altptr->bInterfaceProtocol < device->protocol)
+	  if (altptr->bInterfaceProtocol < device->protocol || altptr->bInterfaceProtocol > 2)
 	    continue;
 
 	  read_endp  = 0xff;
@@ -264,6 +267,8 @@ pappl_usb_find(
 	  }
 	}
 
+        _PAPPL_DEBUG("pappl_usb_find:             device->protocol=%d\n", device->protocol);
+
 	if (device->protocol > 0)
 	{
 	  device->conf  = conf;
@@ -293,7 +298,7 @@ pappl_usb_find(
 	      // Make sure the old, busted usblp kernel driver is not loaded...
 	      if (libusb_kernel_driver_active(device->handle, device->iface) == 1)
 	      {
-		if ((err = libusb_detach_kernel_driver(device->handle, device->iface)) < 0)
+		if ((err = libusb_detach_kernel_driver(device->handle, device->iface)) < 0 && err != LIBUSB_ERROR_NOT_FOUND)
 		{
 		  _papplDeviceError(err_cb, err_data, "Unable to detach usblp kernel driver for USB printer %04x:%04x: %s", devdesc.idVendor, devdesc.idProduct, libusb_strerror((enum libusb_error)err));
 		  libusb_close(device->handle);
@@ -355,7 +360,7 @@ pappl_usb_find(
             if (device->handle)
             {
               // Build the device URI...
-              const char	*make,		// Pointer to make
+              const char *make,		// Pointer to make
 			*model,		// Pointer to model
 			*serial = NULL;	// Pointer to serial number
 	      char	*ptr,		// Pointer into device ID
@@ -473,13 +478,15 @@ pappl_usb_getid(
   _pappl_usb_dev_t	*usb = (_pappl_usb_dev_t *)papplDeviceGetData(device);
 					// USB device data
   size_t		length;		// Length of device ID
-  ssize_t		err;		// Current error
+  int			error;		// USB transfer error
 
 
+  _PAPPL_DEBUG("pappl_usb_getid(device=%p, buffer=%p, bufsize=%ld) usb->conf=%d, ->iface=%d, ->altset=%d\n", device, buffer, (long)bufsize, usb->conf, usb->iface, usb->altset);
+
   // Get the 1284 Device ID...
-  if ((err = libusb_control_transfer(usb->handle, LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_ENDPOINT_IN | LIBUSB_RECIPIENT_INTERFACE, 0, (uint16_t)usb->conf, (uint16_t)((usb->iface << 8) | usb->altset), (unsigned char *)buffer, (uint16_t)bufsize, 5000)) < 0)
+  if ((error = libusb_control_transfer(usb->handle, LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_ENDPOINT_IN | LIBUSB_RECIPIENT_INTERFACE, 0, (uint16_t)usb->conf, (uint16_t)((usb->iface << 8) | usb->altset), (unsigned char *)buffer, (uint16_t)bufsize, 5000)) < 0)
   {
-    papplDeviceError(device, "Unable to get IEEE-1284 device ID: %s", libusb_strerror((enum libusb_error)err));
+    papplDeviceError(device, "Unable to get IEEE-1284 device ID from USB port: %s", libusb_strerror((enum libusb_error)error));
     buffer[0] = '\0';
     return (NULL);
   }
@@ -597,10 +604,17 @@ pappl_usb_read(pappl_device_t *device,	// I - Device
   _pappl_usb_dev_t	*usb = (_pappl_usb_dev_t *)papplDeviceGetData(device);
 					// USB device data
   int			icount;		// Bytes that were read
+  int			error;		// USB transfer error
 
 
-  if (libusb_bulk_transfer(usb->handle, (unsigned char)usb->read_endp, buffer, (int)bytes, &icount, 100) < 0)
+  if (usb->read_endp < 0)
+    return (-1);			// No read endpoint!
+
+  if ((error = libusb_bulk_transfer(usb->handle, (unsigned char)usb->read_endp, buffer, (int)bytes, &icount, 10000)) < 0)
+  {
+    papplDeviceError(device, "Unable to read from USB port: %s",  libusb_strerror((enum libusb_error)error));
     return (-1);
+  }
   else
     return ((ssize_t)icount);
 }
@@ -619,17 +633,22 @@ pappl_usb_status(pappl_device_t *device)// I - Device
 					// IPP "printer-state-reasons" values
   unsigned char		port_status = 0x08;
 					// Centronics port status byte
+  int			error;		// USB transfer error
 
 
-  if (libusb_control_transfer(usb->handle, LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_ENDPOINT_IN | LIBUSB_RECIPIENT_INTERFACE, 1, (uint16_t)usb->conf, (uint16_t)((usb->iface << 8) | usb->altset), &port_status, 1, 5000) >= 0)
+  if ((error = libusb_control_transfer(usb->handle, LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_ENDPOINT_IN | LIBUSB_RECIPIENT_INTERFACE, 1, 0, (uint16_t)(usb->iface << 8), &port_status, 1, 0)) < 0)
+  {
+    papplDeviceError(device, "Unable to get USB port status: %s",  libusb_strerror((enum libusb_error)error));
+  }
+  else
   {
     if (!(port_status & 0x08))
       status |= PAPPL_PREASON_OTHER;
     if (port_status & 0x20)
       status |= PAPPL_PREASON_MEDIA_EMPTY;
-    if (port_status & 0x40)
+    if (port_status & 0x40)		// Vendor extension
       status |= PAPPL_PREASON_MEDIA_JAM;
-    if (port_status & 0x80)
+    if (port_status & 0x80)		// Vendor extension
       status |= PAPPL_PREASON_COVER_OPEN;
   }
 
@@ -638,7 +657,7 @@ pappl_usb_status(pappl_device_t *device)// I - Device
 
 
 //
-// 'pappl_usb_write()' - Write dta to a USB device.
+// 'pappl_usb_write()' - Write data to a USB device.
 //
 
 static ssize_t				// O - Bytes written
@@ -648,11 +667,15 @@ pappl_usb_write(pappl_device_t *device,	// I - Device
 {
   _pappl_usb_dev_t	*usb = (_pappl_usb_dev_t *)papplDeviceGetData(device);
 					// USB device data
-  int	icount;				// Bytes that were written
+  int			icount;		// Bytes that were written
+  int			error;		// USB transfer error
 
 
-  if (libusb_bulk_transfer(usb->handle, (unsigned char)usb->write_endp, (unsigned char *)buffer, (int)bytes, &icount, 0) < 0)
+  if ((error = libusb_bulk_transfer(usb->handle, (unsigned char)usb->write_endp, (unsigned char *)buffer, (int)bytes, &icount, 0)) < 0)
+  {
+    papplDeviceError(device, "Unable to write %d bytes to USB port: %s", (int)bytes, libusb_strerror((enum libusb_error)error));
     return (-1);
+  }
   else
     return ((ssize_t)icount);
 }
