@@ -50,6 +50,7 @@ static cups_array_t	*device_schemes = NULL;
 //
 
 static int		pappl_compare_schemes(_pappl_devscheme_t *a, _pappl_devscheme_t *b);
+static void		pappl_default_error_cb(const char *message, void *data);
 static ssize_t		pappl_write(pappl_device_t *device, const void *buffer, size_t bytes);
 
 
@@ -368,7 +369,7 @@ papplDeviceGetMetrics(
 // The status bits for USB devices come from the original Centronics parallel
 // printer "standard" which was later formally standardized in IEEE 1284-1984
 // and the USB Device Class Definition for Printing Devices.  Some vendor
-// extentions are also supported.
+// extensions are also supported.
 //
 // The status bits for network devices come from the hrPrinterDetectedErrorState
 // property that is defined in the SNMP Printer MIB v2 (RFC 3805).
@@ -506,6 +507,9 @@ papplDeviceList(
 
   pthread_rwlock_rdlock(&device_rwlock);
 
+  if (!err_cb)
+    err_cb = pappl_default_error_cb;
+
   for (ds = (_pappl_devscheme_t *)cupsArrayFirst(device_schemes); ds && !ret; ds = (_pappl_devscheme_t *)cupsArrayNext(device_schemes))
   {
     if ((types & ds->dtype) && ds->list_cb)
@@ -590,8 +594,9 @@ papplDeviceOpen(
   }
 
   device->close_cb   = ds->close_cb;
-  device->error_cb   = err_cb;
+  device->error_cb   = err_cb ? err_cb : pappl_default_error_cb;
   device->error_data = err_data;
+  device->id_cb      = ds->id_cb;
   device->read_cb    = ds->read_cb;
   device->status_cb  = ds->status_cb;
   device->write_cb   = ds->write_cb;
@@ -832,6 +837,21 @@ pappl_compare_schemes(
 }
 
 
+//
+// 'pappl_default_error_cb()' - Send device errors to stderr.
+//
+
+static void
+pappl_default_error_cb(
+    const char *message,		// I - Error message
+    void       *data)			// I - Callback data (unused)
+{
+  (void)data;
+
+  fprintf(stderr, "%s\n", message);
+}
+
+
 //
 // 'pappl_write()' - Write data to the device.
 //
