@@ -27,7 +27,7 @@ static void	start_job(pappl_job_t *job);
 //
 // 'papplJobCreatePrintOptions()' - Create the printer options for a job.
 //
-// This function allocates a printer options stucture and computes the print
+// This function allocates a printer options structure and computes the print
 // options for a job based upon the Job Template attributes submitted in the
 // print request and the default values set in the printer driver data.
 //
@@ -915,8 +915,8 @@ finish_job(pappl_job_t  *job)		// I - Job
     pthread_rwlock_wrlock(&printer->rwlock);
 
     papplDeviceGetMetrics(printer->device, &metrics);
-    papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device read metrics: %lu requests, %lu bytes, %lu msecs", metrics.read_requests, metrics.read_bytes, metrics.read_msecs);
-    papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device write metrics: %lu requests, %lu bytes, %lu msecs", metrics.write_requests, metrics.write_bytes, metrics.write_msecs);
+    papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device read metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.read_requests, (unsigned long)metrics.read_bytes, (unsigned long)metrics.read_msecs);
+    papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device write metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.write_requests, (unsigned long)metrics.write_bytes, (unsigned long)metrics.write_msecs);
 
     papplDeviceClose(printer->device);
     printer->device = NULL;
