@@ -586,11 +586,11 @@ _papplMainloopRunServer(
     // Create the system object...
     system = papplSystemCreate(PAPPL_SOPTIONS_MULTI_QUEUE | PAPPL_SOPTIONS_WEB_INTERFACE, base_name, port, "_print,_universal", directory, logfile, loglevel, cupsGetOption("auth-service", num_options, options), false);
 
-    // Set any admin group and listen for network connections as needed...
+    // Set any admin group and listen for network connections...
     if ((value = cupsGetOption("admin-group", num_options, options)) != NULL)
       papplSystemSetAdminGroup(system, value);
 
-    if (server_name)
+    if (!cupsGetOption("private-server", num_options, options))
       papplSystemAddListeners(system, server_name);
   }
 
@@ -1107,7 +1107,7 @@ _papplMainloopSubmitJob(
 		*supported;		// Supported attributes
   char		default_printer[256],	// Default printer name
 		resource[1024],		// Resource path
-		tempfile[1024];		// Temporary file
+		tempfile[1024] = "";	// Temporary file
   int		i;			// Looping var
   char		*stdin_file;		// Dummy filename for passive stdin jobs
   ipp_attribute_t *job_id;		// job-id for created job
@@ -1223,7 +1223,7 @@ _papplMainloopSubmitJob(
 
     ippDelete(response);
 
-    if (filename == tempfile)
+    if (tempfile[0])
       unlink(tempfile);
   }
 
