@@ -2,7 +2,7 @@
 #
 # Resource generating script for the Printer Application Framework
 #
-# Copyright © 2019-2020 by Michael R Sweet
+# Copyright © 2019-2021 by Michael R Sweet
 #
 # Licensed under Apache License v2.0.  See the file "LICENSE" for more
 # information.
@@ -13,17 +13,17 @@
 #
 
 for file in "$@"; do
-	varname="`echo $file | sed -e '1,$s/[ -.]/_/g'`"
+	varname=$(echo "$file" | sed -e '1,$s/[ -.]/_/g')
 	echo "/* $file */"
 	case $file in
 		*.icc | *.jpg | *.otf | *.otc | *.png | *.ttc | *.ttf | *.woff | *.woff2)
 			echo "static unsigned char $varname[] = {"
-			od -t u1 -A n -v $file | awk '{for (i = 1; i <= NF; i ++) printf("%s,", $i); print "";}'
+			od -t u1 -A n -v "$file" | awk '{for (i = 1; i <= NF; i ++) printf("%s,", $i); print "";}'
 			echo "};"
 			;;
 		*)
 			echo "static const char * const $varname ="
-			sed -e '1,$s/\\/\\\\/g' -e '1,$s/"/\\"/g' $file | awk '{print "\"" $0 "\\n\""}'
+			sed -e '1,$s/\\/\\\\/g' -e '1,$s/"/\\"/g' "$file" | awk '{print "\"" $0 "\\n\""}'
 			echo ";"
 			;;
 	esac
