@@ -20,22 +20,38 @@
 //
 
 static ipp_t	*make_attrs(pappl_system_t *system, pappl_pr_driver_data_t *data);
-static bool	validate_defaults(pappl_printer_t *printer, pappl_pr_driver_data_t *data);
+static bool	validate_defaults(pappl_printer_t *printer, pappl_pr_driver_data_t *driver_data, pappl_pr_driver_data_t *data);
 static bool	validate_driver(pappl_printer_t *printer, pappl_pr_driver_data_t *data);
-static bool	validate_ready(pappl_printer_t *printer, int num_ready, pappl_media_col_t *ready);
+static bool	validate_ready(pappl_printer_t *printer, pappl_pr_driver_data_t *driver_data, int num_ready, pappl_media_col_t *ready);
 
 
 //
-// 'papplPrinterGetDriverAttributes()' - Get the current driver attributes.
+// 'papplPrinterGetDriverAttributes()' - Get a copy of the current driver
+//                                       attributes.
 //
-// This function returns the current driver attributes.
+// This function returns a copy the current driver attributes. Use the
+// `ippDelete` function to free the memory used for the attributes when you
+// are done.
 //
 
-ipp_t *					// O - Driver attributes
+ipp_t *					// O - Copy of driver attributes
 papplPrinterGetDriverAttributes(
     pappl_printer_t *printer)		// I - Printer
 {
-  return (printer ? printer->driver_attrs : NULL);
+  ipp_t	*attrs;				// Copy of driver attributes
+
+
+  if (!printer)
+    return (NULL);
+
+  pthread_rwlock_rdlock(&printer->rwlock);
+
+  attrs = ippNew();
+  ippCopyAttributes(attrs, printer->driver_attrs, 1, NULL, NULL);
+
+  pthread_rwlock_unlock(&printer->rwlock);
+
+  return (attrs);
 }
 
 
@@ -145,7 +161,7 @@ papplPrinterSetDriverData(
     return (false);
 
   // Validate data...
-  if (!validate_defaults(printer, data) || !validate_driver(printer, data) || !validate_ready(printer, data->num_source, data->media_ready))
+  if (!validate_defaults(printer, data, data) || !validate_driver(printer, data) || !validate_ready(printer, data, data->num_source, data->media_ready))
     return (false);
 
   pthread_rwlock_wrlock(&printer->rwlock);
@@ -195,7 +211,7 @@ papplPrinterSetDriverDefaults(
   if (!printer || !data)
     return (false);
 
-  if (!validate_defaults(printer, data))
+  if (!validate_defaults(printer, &printer->driver_data, data))
     return (false);
 
   pthread_rwlock_wrlock(&printer->rwlock);
@@ -284,7 +300,7 @@ papplPrinterSetReadyMedia(
   if (!printer || num_ready <= 0 || !ready)
     return (false);
 
-  if (!validate_ready(printer, num_ready, ready))
+  if (!validate_ready(printer, &printer->driver_data, num_ready, ready))
     return (false);
 
   pthread_rwlock_wrlock(&printer->rwlock);
@@ -1178,7 +1194,8 @@ make_attrs(
 static bool				// O - `true` if valid, `false` otherwise
 validate_defaults(
     pappl_printer_t        *printer,	// I - Printer
-    pappl_pr_driver_data_t *data)	// I - Defaults/supported values
+    pappl_pr_driver_data_t *driver_data,// I - Driver values
+    pappl_pr_driver_data_t *data)	// I - Default values
 {
   bool		ret = true;		// Return value
   int		i;			// Looping var
@@ -1189,22 +1206,22 @@ validate_defaults(
   pwg_media_t	*pwg;			// PWG media size
 
 
-  if (!(data->identify_default & data->identify_supported) && data->identify_supported)
+  if (!(data->identify_default & driver_data->identify_supported) && driver_data->identify_supported)
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported identify-actions-default=0x%04x", data->identify_default);
     ret = false;
   }
-  else if (data->identify_supported)
+  else if (driver_data->identify_supported)
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "identify-actions-default=0x%04x", data->identify_default);
   }
 
-  for (i = 0; i < data->num_media; i ++)
+  for (i = 0; i < driver_data->num_media; i ++)
   {
-    if (!strcmp(data->media[i], data->media_default.size_name))
+    if (!strcmp(driver_data->media[i], data->media_default.size_name))
       break;
 
-    if ((pwg = pwgMediaForPWG(data->media[i])) != NULL)
+    if ((pwg = pwgMediaForPWG(driver_data->media[i])) != NULL)
     {
       if (pwg->width > max_width)
         max_width = pwg->width;
@@ -1218,19 +1235,21 @@ validate_defaults(
     }
   }
 
-  if (i < data->num_media || (data->media_default.size_width >= min_width && data->media_default.size_width <= max_width && data->media_default.size_length >= min_length && data->media_default.size_length <= max_length))
+  if (i < driver_data->num_media || (data->media_default.size_width >= min_width && data->media_default.size_width <= max_width && data->media_default.size_length >= min_length && data->media_default.size_length <= max_length))
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "media-default=%s", data->media_default.size_name);
   }
   else
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-default=%s", data->media_default.size_name);
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "width=%d, length=%d", data->media_default.size_width, data->media_default.size_length);
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "num_media=%d, min_width=%d, max_width=%d, min_length=%d, max_length=%d", driver_data->num_media, min_width, max_width, min_length, max_length);
     ret = false;
   }
 
   papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "orientation-requested-default=%d(%s)", data->orient_default, ippEnumString("orientation-requested", (int)data->orient_default));
 
-  if (!(data->color_default & data->color_supported))
+  if (!(data->color_default & driver_data->color_supported))
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported print-color-mode-default=%s(0x%04x)", _papplColorModeString(data->color_default), data->color_default);
     ret = false;
@@ -1246,12 +1265,12 @@ validate_defaults(
 
   papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "print-scaling-default=%s(0x%04x)", _papplScalingString(data->scaling_default), data->scaling_default);
 
-  for (i = 0; i < data->num_resolution; i ++)
+  for (i = 0; i < driver_data->num_resolution; i ++)
   {
-    if (data->x_default == data->x_resolution[i] && data->y_default == data->y_resolution[i])
+    if (data->x_default == driver_data->x_resolution[i] && data->y_default == driver_data->y_resolution[i])
       break;
   }
-  if (i >= data->num_resolution)
+  if (i >= driver_data->num_resolution)
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported printer-resolution-default=%dx%ddpi", data->x_default, data->y_default);
     ret = false;
@@ -1261,12 +1280,12 @@ validate_defaults(
     papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "printer-resolution-default=%dx%ddpi", data->x_default, data->y_default);
   }
 
-  if (!(data->sides_default & data->sides_supported) && data->sides_supported)
+  if (!(data->sides_default & driver_data->sides_supported) && driver_data->sides_supported)
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported sides-default=%s(0x%04x)", _papplSidesString(data->sides_default), data->sides_default);
     ret = false;
   }
-  else if (data->sides_supported)
+  else if (driver_data->sides_supported)
   {
     papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "sides-default=%s(0x%04x)", _papplSidesString(data->sides_default), data->sides_default);
   }
@@ -1444,14 +1463,102 @@ validate_driver(
 
 static bool				// O - `true` if valid, `false` otherwise
 validate_ready(
-    pappl_printer_t   *printer,		// I - Printer
-    int               num_ready,	// I - Number of ready media values
-    pappl_media_col_t *ready)		// I - Ready media values
+    pappl_printer_t        *printer,	// I - Printer
+    pappl_pr_driver_data_t *driver_data,// I - Driver data
+    int                    num_ready,	// I - Number of ready media values
+    pappl_media_col_t      *ready)	// I - Ready media values
 {
-  // TODO: Validate media-ready values (Issue #94)
-  (void)printer;
-  (void)num_ready;
-  (void)ready;
+  bool		ret = true;		// Return value
+  int		i, j;			// Looping vars
+  int		max_width = 0,		// Maximum media width
+		max_length = 0,		// Maximum media length
+		min_width = 99999999,	// Minimum media width
+		min_length = 99999999;	// Minimum media length
+  pwg_media_t	*pwg;			// PWG media size
 
-  return (true);
+
+  if (num_ready > driver_data->num_source)
+    return (false);
+
+  // Determine the range of media sizes...
+  for (i = 0; i < driver_data->num_media; i ++)
+  {
+    if ((pwg = pwgMediaForPWG(driver_data->media[i])) != NULL)
+    {
+      if (pwg->width > max_width)
+        max_width = pwg->width;
+      if (pwg->width < min_width)
+        min_width = pwg->width;
+
+      if (pwg->length > max_length)
+        max_length = pwg->length;
+      if (pwg->length < min_length)
+        min_length = pwg->length;
+    }
+  }
+
+  for (i = 0; i < num_ready; i ++)
+  {
+    if (!ready[i].size_name[0])
+      continue;
+
+    if (!pwgMediaForPWG(ready[i].size_name))
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Invalid media-ready.media-size-name='%s'.", ready[i].size_name);
+      ret = false;
+    }
+    else if (ready[i].size_width < min_width || ready[i].size_width > max_width || ready[i].size_length < min_length || ready[i].size_length > max_length)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-size=%.2fx%.2fmm.", ready[i].size_width * 0.01, ready[i].size_length * 0.01);
+      ret = false;
+    }
+
+    if (ready[i].left_margin < driver_data->left_right && !driver_data->borderless)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-left-margin=%d.", ready[i].left_margin);
+      ret = false;
+    }
+
+    if (ready[i].right_margin < driver_data->left_right && !driver_data->borderless)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-right-margin=%d.", ready[i].right_margin);
+      ret = false;
+    }
+
+    if (ready[i].top_margin < driver_data->bottom_top && !driver_data->borderless)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-top-margin=%d.", ready[i].top_margin);
+      ret = false;
+    }
+
+    if (ready[i].bottom_margin < driver_data->bottom_top && !driver_data->borderless)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-bottom-margin=%d.", ready[i].bottom_margin);
+      ret = false;
+    }
+
+    for (j = 0; j < driver_data->num_source; j ++)
+    {
+      if (!strcmp(ready[i].source, driver_data->source[j]))
+        break;
+    }
+    if (j >= driver_data->num_source)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-source='%s'.", ready[i].source);
+      ret = false;
+    }
+
+    for (j = 0; j < driver_data->num_type; j ++)
+    {
+      if (!strcmp(ready[i].type, driver_data->type[j]))
+        break;
+    }
+    if (j >= driver_data->num_type)
+    {
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unsupported media-ready.media-type='%s'.", ready[i].type);
+      ret = false;
+    }
+  }
+
+  return (ret);
 }
