@@ -111,8 +111,8 @@ _papplPrinterWebCancelJob(
   int		job_id = 0;             // Job ID to cancel
   pappl_job_t	*job;			// Job to cancel
   const char	*status = NULL;		// Status message, if any
-  int		num_form = 0;		// Number of form variables
-  cups_option_t	*form = NULL;		// Form variables
+  int		num_form;		// Number of form variables
+  cups_option_t	*form;			// Form variables
   const char	*value;			// Value of form variable
 
 
@@ -121,7 +121,6 @@ _papplPrinterWebCancelJob(
 
   if (client->operation == HTTP_STATE_GET)
   {
-
     if ((num_form = papplClientGetForm(client, &form)) == 0)
     {
       status = "Invalid GET data.";
@@ -175,13 +174,13 @@ _papplPrinterWebCancelJob(
       {
         status = "Invalid Job ID.";
       }
+
+      cupsFreeOptions(num_form, form);
     }
     else
     {
       status = "Invalid form submission.";
     }
-
-    cupsFreeOptions(num_form, form);
   }
 
   papplClientHTMLPrinterHeader(client, printer, "Cancel Job", 0, NULL, NULL);
@@ -462,11 +461,13 @@ _papplPrinterWebDefaults(
 	  num_vendor = cupsAddOption(data.vendor[i], "false", num_vendor, &vendor);
       }
 
-      papplPrinterSetDriverDefaults(printer, &data, num_vendor, vendor);
+      if (papplPrinterSetDriverDefaults(printer, &data, num_vendor, vendor))
+        status = "Changes saved.";
+      else
+        status = "Bad printer defaults.";
 
       cupsFreeOptions(num_vendor, vendor);
 
-      status = "Changes saved.";
     }
 
     cupsFreeOptions(num_form, form);
@@ -483,30 +484,40 @@ _papplPrinterWebDefaults(
 		      "            <tbody>\n");
 
   // media-col-default
-  papplClientHTMLPuts(client, "              <tr><th>Media:</th><td><select name=\"media-source\">");
-  for (i = 0; i < data.num_source; i ++)
+  papplClientHTMLPuts(client, "              <tr><th>Media:</th><td>");
+
+  if (data.num_source > 1)
   {
-    // See if any two sources have the same size...
-    for (j = i + 1; j < data.num_source; j ++)
+    papplClientHTMLPuts(client, "<select name=\"media-source\">");
+
+    for (i = 0; i < data.num_source; i ++)
     {
-      if (data.media_ready[i].size_width > 0 && data.media_ready[i].size_width == data.media_ready[j].size_width && data.media_ready[i].size_length == data.media_ready[j].size_length)
+      // See if any two sources have the same size...
+      for (j = i + 1; j < data.num_source; j ++)
       {
-        show_source = true;
-        break;
+	if (data.media_ready[i].size_width > 0 && data.media_ready[i].size_width == data.media_ready[j].size_width && data.media_ready[i].size_length == data.media_ready[j].size_length)
+	{
+	  show_source = true;
+	  break;
+	}
       }
     }
-  }
 
-  for (i = 0; i < data.num_source; i ++)
-  {
-    keyword = data.source[i];
-
-    if (strcmp(keyword, "manual"))
+    for (i = 0; i < data.num_source; i ++)
     {
-      papplClientHTMLPrintf(client, "<option value=\"%s\"%s>%s</option>", keyword, !strcmp(keyword, data.media_default.source) ? " selected" : "", localize_media(data.media_ready + i, show_source, text, sizeof(text)));
+      keyword = data.source[i];
+
+      if (strcmp(keyword, "manual"))
+      {
+	papplClientHTMLPrintf(client, "<option value=\"%s\"%s>%s</option>", keyword, !strcmp(keyword, data.media_default.source) ? " selected" : "", localize_media(data.media_ready + i, show_source, text, sizeof(text)));
+      }
     }
+    papplClientHTMLPuts(client, "</select>");
   }
-  papplClientHTMLPrintf(client, "</select> <a class=\"btn\" href=\"%s/media\">Configure Media</a></td></tr>\n", printer->uriname);
+  else
+    papplClientHTMLEscape(client, localize_media(data.media_ready, false, text, sizeof(text)), 0);
+
+  papplClientHTMLPrintf(client, " <a class=\"btn\" href=\"%s/media\">Configure Media</a></td></tr>\n", printer->uriname);
 
   // orientation-requested-default
   papplClientHTMLPuts(client, "              <tr><th>Orientation:</th><td>");
@@ -1552,9 +1563,10 @@ localize_media(
     char              *buffer,		// I - String buffer
     size_t            bufsize)		// I - Size of string buffer
 {
-  char		size[128],		// I - Size name string
-		source[128],		// I - Source string
-		type[128];		// I - Type string
+  char		size[128],		// Size name string
+		source[128],		// Source string
+		type[128];		// Type string
+  const char	*borderless;		// Borderless qualifier
 
 
   if (!media->size_name[0])
@@ -1567,10 +1579,15 @@ localize_media(
   else
     localize_keyword("media-type", media->type, type, sizeof(type));
 
-  if (include_source)
-    snprintf(buffer, bufsize, "%s (%s) from %s", size, type, localize_keyword("media-source", media->source, source, sizeof(source)));
+  if (!media->left_margin && !media->right_margin && !media->top_margin && !media->bottom_margin)
+    borderless = ", Borderless";
   else
-    snprintf(buffer, bufsize, "%s (%s)", size, type);
+    borderless = "";
+
+  if (include_source)
+    snprintf(buffer, bufsize, "%s (%s%s) from %s", size, type, borderless, localize_keyword("media-source", media->source, source, sizeof(source)));
+  else
+    snprintf(buffer, bufsize, "%s (%s%s)", size, type, borderless);
 
   return (buffer);
 }
