@@ -39,7 +39,9 @@ papplPrinterCancelAllJobs(
   pappl_job_t	*job;			// Job information
 
 
-  // Loop through all jobs and cancel them...
+  // Loop through all jobs and cancel them.
+  //
+  // Since we have a writer lock, it is safe to use cupsArrayFirst/Last...
   pthread_rwlock_wrlock(&printer->rwlock);
 
   for (job = (pappl_job_t *)cupsArrayFirst(printer->active_jobs); job; job = (pappl_job_t *)cupsArrayNext(printer->active_jobs))
@@ -199,7 +201,7 @@ papplPrinterCreate(
 
 
   // Range check input...
-  if (!system || !printer_name || !driver_name || !device_uri || !strcmp(printer_name, "ipp"))
+  if (!system || !printer_name || !driver_name || !device_uri)
   {
     errno = EINVAL;
     return (NULL);
@@ -215,20 +217,64 @@ papplPrinterCreate(
   // Prepare URI values for the printer attributes...
   if (system->options & PAPPL_SOPTIONS_MULTI_QUEUE)
   {
-    snprintf(resource, sizeof(resource), "/ipp/print/%s", printer_name);
+    // Make sure printer names that start with a digit have a resource path
+    // containing an underscore...
+    if (isdigit(*printer_name & 255))
+      snprintf(resource, sizeof(resource), "/ipp/print/_%s", printer_name);
+    else
+      snprintf(resource, sizeof(resource), "/ipp/print/%s", printer_name);
+
+    // Convert URL reserved characters to underscore...
     for (resptr = resource + 11; *resptr; resptr ++)
-      if ((*resptr & 255) <= ' ' || *resptr == 0x7f)
+    {
+      if ((*resptr & 255) <= ' ' || strchr("\177/\\\'\"?#", *resptr))
 	*resptr = '_';
+    }
+
+    // Eliminate duplicate and trailing underscores...
+    resptr = resource + 11;
+    while (*resptr)
+    {
+      if (resptr[0] == '_' && resptr[1] == '_')
+        memmove(resptr, resptr + 1, strlen(resptr));
+					// Duplicate underscores
+      else if (resptr[0] == '_' && !resptr[1])
+        *resptr = '\0';			// Trailing underscore
+      else
+        resptr ++;
+    }
   }
   else
     strlcpy(resource, "/ipp/print", sizeof(resource));
 
   // Make sure the printer doesn't already exist...
-  if (papplSystemFindPrinter(system, resource, 0, NULL))
+  if ((printer = papplSystemFindPrinter(system, resource, 0, NULL)) != NULL)
   {
-    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Printer '%s' already exists.", printer_name);
-    errno = EEXIST;
-    return (NULL);
+    int		n;		// Current instance number
+    char	temp[1024];	// Temporary resource path
+
+    if (!strcmp(printer_name, printer->name))
+    {
+      papplLog(system, PAPPL_LOGLEVEL_ERROR, "Printer '%s' already exists.", printer_name);
+      errno = EEXIST;
+      return (NULL);
+    }
+
+    for (n = 2; n < 10; n ++)
+    {
+      snprintf(temp, sizeof(temp), "%s_%d", resource, n);
+      if (!papplSystemFindPrinter(system, temp, 0, NULL))
+        break;
+    }
+
+    if (n >= 10)
+    {
+      papplLog(system, PAPPL_LOGLEVEL_ERROR, "Printer '%s' name conflicts with existing printer.", printer_name);
+      errno = EEXIST;
+      return (NULL);
+    }
+
+    strlcpy(resource, temp, sizeof(resource));
   }
 
   // Allocate memory for the printer...
@@ -292,7 +338,23 @@ papplPrinterCreate(
   // If the driver is "auto", figure out the proper driver name...
   if (!strcmp(driver_name, "auto") && system->autoadd_cb)
   {
-    if ((driver_name = (system->autoadd_cb)(printer_name, device_uri, device_id, system->driver_cbdata)) == NULL)
+    // If device_id is NULL, try to look it up...
+    if (!printer->device_id && strncmp(device_uri, "file://", 7))
+    {
+      pappl_device_t	*device;	// Connection to printer
+
+      if ((device = papplDeviceOpen(device_uri, "auto", papplLogDevice, system)) != NULL)
+      {
+        char	new_id[1024];		// New 1284 device ID
+
+        if (papplDeviceGetID(device, new_id, sizeof(new_id)))
+          printer->device_id = strdup(new_id);
+
+        papplDeviceClose(device);
+      }
+    }
+
+    if ((driver_name = (system->autoadd_cb)(printer_name, device_uri, printer->device_id, system->driver_cbdata)) == NULL)
     {
       errno = EIO;
       _papplPrinterDelete(printer);
@@ -613,9 +675,46 @@ void
 _papplPrinterDelete(
     pappl_printer_t *printer)		// I - Printer
 {
+  int			i;		// Looping var
+  _pappl_resource_t	*r;		// Current resource
+  char			prefix[1024];	// Prefix for printer resources
+  size_t		prefixlen;	// Length of prefix
+
+
+  // Let USB/raw printing threads know to exit
+  printer->is_deleted = true;
+
+  while (printer->raw_active || printer->usb_active)
+  {
+    // Wait for threads to finish
+    usleep(100000);
+  }
+
+  // Close raw listener sockets...
+  for (i = 0; i < printer->num_raw_listeners; i ++)
+  {
+    close(printer->raw_listeners[i].fd);
+    printer->raw_listeners[i].fd = -1;
+  }
+
+  printer->num_raw_listeners = 0;
+
   // Remove DNS-SD registrations...
   _papplPrinterUnregisterDNSSDNoLock(printer);
 
+  // Remove printer-specific resources...
+  snprintf(prefix, sizeof(prefix), "%s/", printer->uriname);
+  prefixlen = strlen(prefix);
+
+  // Note: System writer lock is already held when calling cupsArrayRemove
+  // for the system's printer object, so we don't need a separate lock here
+  // and can safely use cupsArrayFirst/Next...
+  for (r = (_pappl_resource_t *)cupsArrayFirst(printer->system->resources); r; r = (_pappl_resource_t *)cupsArrayNext(printer->system->resources))
+  {
+    if (r->cbdata == printer || !strncmp(r->path, prefix, prefixlen))
+      cupsArrayRemove(printer->system->resources, r);
+  }
+
   // If applicable, call the delete function...
   if (printer->driver_data.delete_cb)
     (printer->driver_data.delete_cb)(printer, &printer->driver_data);
