@@ -1,7 +1,7 @@
 //
 // System accessor functions for the Printer Application Framework
 //
-// Copyright © 2020 by Michael R Sweet.
+// Copyright © 2020-2021 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -1082,15 +1082,21 @@ papplSystemIteratePrinters(
     pappl_printer_cb_t cb,		// I - Callback function
     void               *data)		// I - Callback data
 {
-  pappl_printer_t	*printer;	// Current printer
+  int			i,		// Looping var
+			count;		// Number of printers
 
 
   if (!system || !cb)
     return;
 
+  // Loop through the printers.
+  //
+  // Note: Cannot use cupsArrayFirst/Last since other threads might be
+  // enumerating the printers array.
+
   pthread_rwlock_rdlock(&system->rwlock);
-  for (printer = (pappl_printer_t *)cupsArrayFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayNext(system->printers))
-    (cb)(printer, data);
+  for (i = 0, count = cupsArrayCount(system->printers); i < count; i ++)
+    (cb)((pappl_printer_t *)cupsArrayIndex(system->printers, i), data);
   pthread_rwlock_unlock(&system->rwlock);
 }
 
@@ -1212,8 +1218,7 @@ papplSystemSetAdminGroup(
     else
       system->admin_gid = (gid_t)-1;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1238,8 +1243,7 @@ papplSystemSetContact(
 
   system->contact = *contact;
 
-  system->config_time = time(NULL);
-  system->config_changes ++;
+  _papplSystemConfigChanged(system);
 
   pthread_rwlock_unlock(&system->rwlock);
 }
@@ -1263,8 +1267,7 @@ papplSystemSetDefaultPrinterID(
 
     system->default_printer_id = default_printer_id;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1292,8 +1295,7 @@ papplSystemSetDefaultPrintGroup(
     free(system->default_print_group);
     system->default_print_group = value ? strdup(value) : NULL;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1320,8 +1322,8 @@ papplSystemSetDNSSDName(
     system->dns_sd_name      = value ? strdup(value) : NULL;
     system->dns_sd_collision = false;
     system->dns_sd_serial    = 0;
-    system->config_time      = time(NULL);
-    system->config_changes ++;
+
+    _papplSystemConfigChanged(system);
 
     if (!value)
       _papplSystemUnregisterDNSSDNoLock(system);
@@ -1371,14 +1373,21 @@ papplSystemSetGeoLocation(
     pappl_system_t *system,		// I - System
     const char     *value)		// I - "geo:" URI or `NULL` for none
 {
+  float	lat, lon;			// Latitude and longitude from geo: URI
+
+
+  // Validate geo-location - must be NULL or a "geo:" URI...
+  if (value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
+    return;
+
   if (system)
   {
     pthread_rwlock_wrlock(&system->rwlock);
 
     free(system->geo_location);
     system->geo_location = value ? strdup(value) : NULL;
-    system->config_time  = time(NULL);
-    system->config_changes ++;
+
+    _papplSystemConfigChanged(system);
 
     _papplSystemRegisterDNSSDNoLock(system);
 
@@ -1421,7 +1430,8 @@ papplSystemSetHostname(
       _pappl_dns_sd_t	master = _papplDNSSDInit(system);
 					  // DNS-SD master reference
 
-      avahi_client_set_host_name(master, value);
+      if (master)
+        avahi_client_set_host_name(master, value);
 #endif // HAVE_AVAHI
 
       sethostname(value, (int)strlen(value));
@@ -1436,7 +1446,7 @@ papplSystemSetHostname(
 #ifdef HAVE_AVAHI
       _pappl_dns_sd_t	master = _papplDNSSDInit(system);
 					  // DNS-SD master reference
-      const char *avahi_name = avahi_client_get_host_name_fqdn(master);
+      const char *avahi_name = master ? avahi_client_get_host_name_fqdn(master) : NULL;
 					  // mDNS hostname
 
       if (avahi_name)
@@ -1486,8 +1496,8 @@ papplSystemSetLocation(
 
     free(system->location);
     system->location    = value ? strdup(value) : NULL;
-    system->config_time = time(NULL);
-    system->config_changes ++;
+
+    _papplSystemConfigChanged(system);
 
     _papplSystemRegisterDNSSDNoLock(system);
 
@@ -1512,8 +1522,7 @@ papplSystemSetLogLevel(
 
     system->loglevel = loglevel;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1541,8 +1550,7 @@ papplSystemSetMaxLogSize(
 
     system->logmaxsize = maxsize;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1604,8 +1612,7 @@ papplSystemSetNextPrinterID(
 
     system->next_printer_id = next_printer_id;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1658,8 +1665,7 @@ papplSystemSetOrganization(
     free(system->organization);
     system->organization = value ? strdup(value) : NULL;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1686,8 +1692,7 @@ papplSystemSetOrganizationalUnit(
     free(system->org_unit);
     system->org_unit = value ? strdup(value) : NULL;
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1715,8 +1720,7 @@ papplSystemSetPassword(
 
     strlcpy(system->password_hash, hash, sizeof(system->password_hash));
 
-    system->config_time = time(NULL);
-    system->config_changes ++;
+    _papplSystemConfigChanged(system);
 
     pthread_rwlock_unlock(&system->rwlock);
   }
@@ -1808,7 +1812,7 @@ papplSystemSetSaveCallback(
 //
 // 'papplSystemSetUUID()' - Set the system UUID.
 //
-// This function sets the system UUID value, overridding the default (generated)
+// This function sets the system UUID value, overriding the default (generated)
 // value.  It is typically used when restoring the state of a previous
 // incarnation of the system.
 //
@@ -1851,9 +1855,6 @@ papplSystemSetUUID(
 // This function sets the names and versions of each firmware/software component
 // of the printer application.
 //
-// > Note: The firmware information can only be set prior to calling
-// > @link papplSystemRun@.
-//
 
 void
 papplSystemSetVersions(
@@ -1861,7 +1862,7 @@ papplSystemSetVersions(
     int             num_versions,	// I - Number of versions
     pappl_version_t *versions)		// I - Firmware versions
 {
-  if (system && num_versions && versions && !system->is_running)
+  if (system && num_versions && versions)
   {
     pthread_rwlock_wrlock(&system->rwlock);
 
