@@ -21,7 +21,7 @@
 // Local globals...
 //
 
-static bool	shutdown_system = false;// Shutdown system?
+static time_t	sigterm_time = 0;	// SIGTERM time?
 static bool	restart_logging = false;// Restart logging?
 
 
@@ -85,12 +85,15 @@ void
 _papplSystemConfigChanged(
     pappl_system_t *system)		// I - System
 {
-  pthread_rwlock_wrlock(&system->rwlock);
+  pthread_mutex_lock(&system->config_mutex);
 
   if (system->is_running)
+  {
+    system->config_time = time(NULL);
     system->config_changes ++;
+  }
 
-  pthread_rwlock_unlock(&system->rwlock);
+  pthread_mutex_unlock(&system->config_mutex);
 }
 
 
@@ -170,6 +173,7 @@ papplSystemCreate(
   // Initialize values...
   pthread_rwlock_init(&system->rwlock, NULL);
   pthread_rwlock_init(&system->session_rwlock, NULL);
+  pthread_mutex_init(&system->config_mutex, NULL);
 
   system->options         = options;
   system->start_time      = time(NULL);
@@ -308,6 +312,7 @@ papplSystemDelete(
 
   pthread_rwlock_destroy(&system->rwlock);
   pthread_rwlock_destroy(&system->session_rwlock);
+  pthread_mutex_destroy(&system->config_mutex);
 
   free(system);
 }
@@ -371,6 +376,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
   int			dns_sd_host_changes;
 					// Current number of host name changes
   pappl_printer_t	*printer;	// Current printer
+  pappl_printer_t	*usb_printer = NULL;
+					// USB printer
 
 
   // Range check...
@@ -484,14 +491,16 @@ papplSystemRun(pappl_system_t *system)	// I - System
     _papplSystemRegisterDNSSDNoLock(system);
 
   // Start up printers...
-  for (printer = (pappl_printer_t *)cupsArrayFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayNext(system->printers))
+  for (i = 0, count = cupsArrayCount(system->printers); i < count; i ++)
   {
+    printer = (pappl_printer_t *)cupsArrayIndex(system->printers, i);
+
     // Advertise via DNS-SD as needed...
     if (printer->dns_sd_name)
       _papplPrinterRegisterDNSSDNoLock(printer);
 
     // Start the raw socket listeners as needed...
-    if ((system->options & PAPPL_SOPTIONS_RAW_SOCKET) && printer->num_listeners > 0)
+    if ((system->options & PAPPL_SOPTIONS_RAW_SOCKET) && printer->num_raw_listeners > 0)
     {
       pthread_t	tid;		// Thread ID
 
@@ -512,11 +521,11 @@ papplSystemRun(pappl_system_t *system)	// I - System
   if (system->options & PAPPL_SOPTIONS_USB_PRINTER)
   {
     // USB support is limited to a single (default) printer...
-    if ((printer = papplSystemFindPrinter(system, NULL, system->default_printer_id, NULL)) != NULL)
+    if ((usb_printer = papplSystemFindPrinter(system, NULL, system->default_printer_id, NULL)) != NULL)
     {
       pthread_t	tid;			// Thread ID
 
-      if (pthread_create(&tid, NULL, (void *(*)(void *))_papplPrinterRunUSB, printer))
+      if (pthread_create(&tid, NULL, (void *(*)(void *))_papplPrinterRunUSB, usb_printer))
       {
 	// Unable to create USB thread...
 	papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create USB printer thread: %s", strerror(errno));
@@ -530,7 +539,7 @@ papplSystemRun(pappl_system_t *system)	// I - System
   }
 
   // Loop until we are shutdown or have a hard error...
-  while (!shutdown_system)
+  for (;;)
   {
     if (restart_logging)
     {
@@ -585,8 +594,10 @@ papplSystemRun(pappl_system_t *system)	// I - System
       if (system->dns_sd_collision || force_dns_sd)
         _papplSystemRegisterDNSSDNoLock(system);
 
-      for (printer = (pappl_printer_t *)cupsArrayFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayNext(system->printers))
+      for (i = 0, count = cupsArrayCount(system->printers); i < count; i ++)
       {
+	printer = (pappl_printer_t *)cupsArrayIndex(system->printers, i);
+
         if (printer->dns_sd_collision || force_dns_sd)
           _papplPrinterRegisterDNSSDNoLock(printer);
       }
@@ -599,8 +610,12 @@ papplSystemRun(pappl_system_t *system)	// I - System
 
     if (system->config_changes > system->save_changes)
     {
+      pthread_mutex_lock(&system->config_mutex);
+
       system->save_changes = system->config_changes;
 
+      pthread_mutex_unlock(&system->config_mutex);
+
       if (system->save_cb)
       {
         // Save the configuration...
@@ -608,19 +623,24 @@ papplSystemRun(pappl_system_t *system)	// I - System
       }
     }
 
-    if (system->shutdown_time)
+    if (system->shutdown_time || sigterm_time)
     {
       // Shutdown requested, see if we can do so safely...
       int		jcount = 0;	// Number of active jobs
 
       // Force shutdown after 60 seconds
-      if ((time(NULL) - system->shutdown_time) > 60)
-        break;
+      if (system->shutdown_time && (time(NULL) - system->shutdown_time) > 60)
+        break;				// Shutdown-System request
+
+      if (sigterm_time && (time(NULL) - sigterm_time) > 60)
+        break;				// SIGTERM received
 
       // Otherwise shutdown immediately if there are no more active jobs...
       pthread_rwlock_rdlock(&system->rwlock);
-      for (printer = (pappl_printer_t *)cupsArrayFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayNext(system->printers))
+      for (i = 0, count = cupsArrayCount(system->printers); i < count; i ++)
       {
+	printer = (pappl_printer_t *)cupsArrayIndex(system->printers, i);
+
         pthread_rwlock_rdlock(&printer->rwlock);
         jcount += cupsArrayCount(printer->active_jobs);
         pthread_rwlock_unlock(&printer->rwlock);
@@ -644,9 +664,11 @@ papplSystemRun(pappl_system_t *system)	// I - System
   if (system->dns_sd_name)
     _papplSystemUnregisterDNSSDNoLock(system);
 
-  for (printer = (pappl_printer_t *)cupsArrayFirst(system->printers); printer; printer = (pappl_printer_t *)cupsArrayNext(system->printers))
+  for (i = 0, count = cupsArrayCount(system->printers); i < count; i ++)
   {
-    // Advertise via DNS-SD as needed...
+    printer = (pappl_printer_t *)cupsArrayIndex(system->printers, i);
+
+    // Remove advertising via DNS-SD as needed...
     if (printer->dns_sd_name)
       _papplPrinterUnregisterDNSSDNoLock(printer);
   }
@@ -659,8 +681,11 @@ papplSystemRun(pappl_system_t *system)	// I - System
 
   system->is_running = false;
 
-  if (system->options & PAPPL_SOPTIONS_USB_PRINTER)
-    sleep(2);
+  if (usb_printer)
+  {
+    while (usb_printer->usb_active)
+      usleep(100000);
+  }
 }
 
 
@@ -784,5 +809,5 @@ sigterm_handler(int sig)		// I - Signal (ignored)
 {
   (void)sig;
 
-  shutdown_system = true;
+  sigterm_time = time(NULL);
 }
